% NF-kB Signaling Model version 2.1
%
% Werner, S.L., Barken, D., Hoffmann, A. 
%  Stimulus-specificity of gene expression programs determined 
%  by temporal control of IKK activity. 
% Science, 309, pp.1857-61. 2005.
%
% Signaling Systems Laboratory (PI: Dr. Alexander Hoffmann) 
% Department of Chemistry and Biochemistry 
% University of California at San Diego
% 
% Questions should be addressed to
% Dr. Alexander Hoffmann    ahoffmann@ucsd.edu
%
%==========================================================
% Copyright 2007   The Regents of the University of California 
% All Rights Reserved 
%
% Permission to use, copy, modify and distribute any part of this
% NF-kB Signaling Model version 2.1 software for educational, research and
% non-profit purposes, without fee, and without a written agreement is
% hereby granted, provided that the above citation and copyright notice, this 
% paragraph and the following three paragraphs appear in all copies.
% 
% Those desiring to incorporate this NF-kB Signaling Model version 2.1
% software into commercial products or use for commercial purposes should
% contact the Technology Transfer & Intellectual Property Services,
% University of California, San Diego, 9500 Gilman Drive, Mail Code 0910,
% La Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345,
% E-MAIL:invent@ucsd.edu.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
% INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS NF-KB SIGNALING
% MODEL VERSION 2.1 SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
% ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% THE NF-KB SIGNALING MODEL VERSION 2.1 SOFTWARE PROVIDED HEREIN IS ON AN
% "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
% PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
% THE UNIVERSITY OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO
% WARRANTIES OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
% PARTICULAR PURPOSE, OR THAT THE USE OF THE NF-KB SIGNALING MODEL VERSION
% 2.1 SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.
%==========================================================

% ODE Function

function delta = nfkb_ode_21pr(t,x)

    % Global Parameters
    global CURRENT_IKK;
    global GP;             
    global PHASE;
    global HISTORY_TIME;
    global HISTORY_NFKBN;

%==========================================================
% Set concentrations
    [a,b] = size(x);
    delta = zeros(a,b);
    
    IkBa           = x(1);
    IkBaIKK        = x(2);
    IkBaIKKNFkB    = x(3);
    IkBan          = x(4);
    IkBaNFkB       = x(5);
    IkBaNFkBn      = x(6);
    IkBat          = x(7);
    IkBb           = x(8);
    IkBbIKK        = x(9);
    IkBbIKKNFkB    = x(10);
    IkBbn          = x(11);
    IkBbNFkB       = x(12);
    IkBbNFkBn      = x(13);
    IkBbt          = x(14);
    IkBe           = x(15);
    IkBeIKK        = x(16);
    IkBeIKKNFkB    = x(17);
    IkBen          = x(18);
    IkBeNFkB       = x(19);
    IkBeNFkBn      = x(20);
    IkBet          = x(21);
    IKK            = x(22);
    NFkB           = x(23);
    NFkBn          = x(24);
    p100t          = x(25);
    p100           = x(26);
    p52            = x(27);
    NIK            = x(28);
    RelB           = x(29);      % RelB
    RelBp52        = x(30);      % RelBp52
    RelBnp52n      = x(31);      % RelBnp52n
    RelBt           = x(32);      % RelBnp52n

    


%==========================================================
% Initialize Fluxes (all set to zero)
    delta_IkBa        = 0;
    delta_IkBaIKK     = 0;
    delta_IkBaIKKNFkB = 0;
    delta_IkBan       = 0;
    delta_IkBaNFkB    = 0;
    delta_IkBaNFkBn   = 0;
    delta_IkBat       = 0;
    delta_IkBb        = 0;
    delta_IkBbIKK     = 0;
    delta_IkBbIKKNFkB = 0;
    delta_IkBbn       = 0;
    delta_IkBbNFkB    = 0;
    delta_IkBbNFkBn   = 0;
    delta_IkBbt       = 0;
    delta_IkBe        = 0;
    delta_IkBeIKK     = 0;
    delta_IkBeIKKNFkB = 0;
    delta_IkBen       = 0;
    delta_IkBeNFkB    = 0;
    delta_IkBeNFkBn   = 0;
    delta_IkBet       = 0;
    delta_IKK         = 0;
    delta_NFkB        = 0;
    delta_NFkBn       = 0;
    delta_p100t       = 0;
    delta_p100        = 0;
    delta_p52         = 0;
    delta_NIK         = 0;
    delta_RelB        = 0;
    delta_RelBp52     = 0;
    delta_RelBnp52n   = 0;
    delta_RelBt       = 0;

%==========================================================
% Save NFkBn Values for Induced Transcription
%       The find command is needed to prevent duplicate values
%       that break the interpolation
if PHASE == 2
     if (isempty(HISTORY_TIME) || (isempty(find(HISTORY_TIME == t,1))) )
         if (isempty(HISTORY_NFKBN) || (isempty(find(HISTORY_NFKBN == NFkBn,1))) )
             HISTORY_NFKBN(end+1,1) = NFkBn;
             HISTORY_TIME(end+1,1) = t; 
         end
     end
end

%==========================================================
% Calculate Delayed NFkBn Values for Induced Transcription

delayed_nfkbn_a = NFkBn;  % ikba delay is not enabled
delayed_nfkbn_b = NFkBn;  
delayed_nfkbn_e = NFkBn;

if PHASE == 2  % Delay is only in Phase 2    
    if GP(8) > 0  % IkBb inducible txn delay
        if t > GP(8) 
            delayed_nfkbn_b = interp1(HISTORY_TIME,HISTORY_NFKBN,t-GP(8));
        else
            delayed_nfkbn_b = HISTORY_NFKBN(1); % first index  is t =0  
        end
    end

    if GP(9) > 0  % IkBe inducible txn delay
        if t > GP(9) 
            if GP(9) == GP(8)  % saves cpu time if IkBb/IkBe delays are equal
                delayed_nfkbn_e = delayed_nfkbn_b;
            else
                delayed_nfkbn_e = interp1(HISTORY_TIME,HISTORY_NFKBN,t-GP(9));

            end
        else
            delayed_nfkbn_e   = HISTORY_NFKBN(1); % first index  is t =0  
        end
    end
    
end

%==========================================================
% Calculate IKK Activity (used in IKK-mediated deg reaction)

        if PHASE == 1
            ikk_multiplier  = CURRENT_IKK(1);    % 1 percent at basal  
        else % Phase = 2
            ikk_multiplier  = CURRENT_IKK(t);
        end 
        
%==========================================================
% Calculate Fluxes for Each Reaction
    flux_rsu_a          = GP(1) ;
    flux_rsu_b          = GP(2) ;
    flux_rsu_e          = GP(3) ;

    flux_rsr_an         = GP(4) * (delayed_nfkbn_a ^ GP(10)) ;
    flux_rsr_bn         = GP(5) * (delayed_nfkbn_b ^ GP(11)) ;
    flux_rsr_en         = GP(6) * (delayed_nfkbn_e ^ GP(12)) ;
      
    flux_rd_a           = GP(13)* IkBat	;
    flux_rd_b           = GP(14)* IkBbt	;
    flux_rd_e           = GP(15)* IkBet	;
      
    flux_ps_c_a         = GP(16)* IkBat	;
    flux_ps_c_b         = GP(17)* IkBbt	;
    flux_ps_c_e         = GP(18)* IkBet	;

    flux_a_c_an         = GP(19)* IkBa	* NFkB ;
    flux_a_c_bn         = GP(20)* IkBb	* NFkB ;   
    flux_a_c_en         = GP(21)* IkBe	* NFkB ;   
          
    flux_a_n_an         = GP(22)* IkBan	* NFkBn	;
    flux_a_n_bn         = GP(23)* IkBbn	* NFkBn	;
    flux_a_n_en         = GP(24)* IkBen	* NFkBn	;      
    
    flux_d_c_an         = GP(25)* IkBaNFkB ;
    flux_d_c_bn         = GP(26)* IkBbNFkB ;
    flux_d_c_en         = GP(27)* IkBeNFkB ;
    
    flux_d_n_an         = GP(28)* IkBaNFkBn	;
    flux_d_n_bn         = GP(29)* IkBbNFkBn	;
    flux_d_n_en         = GP(30)* IkBeNFkBn	;
    
    flux_a_c_ai         = GP(31)* IkBa	* IKK ;
    flux_a_c_bi         = GP(32)* IkBb	* IKK ;
    flux_a_c_ei         = GP(33)* IkBe	* IKK ;
    
    flux_d_c_ai         = GP(34)* IkBaIKK ;
    flux_d_c_bi         = GP(35)* IkBbIKK ;
    flux_d_c_ei         = GP(36)* IkBeIKK ;
    
    flux_a_c_2ani       = GP(37)* IkBaNFkB	* IKK ;
    flux_a_c_2bni       = GP(38)* IkBbNFkB	* IKK ;
    flux_a_c_2eni       = GP(39)* IkBeNFkB	* IKK ;
    
    flux_d_c_2ani       = GP(40)* IkBaIKKNFkB ;
    flux_d_c_2bni       = GP(41)* IkBbIKKNFkB ;
    flux_d_c_2eni       = GP(42)* IkBeIKKNFkB ;
    
    flux_a_c_2ain       = GP(43)* IkBaIKK	* NFkB ;
    flux_a_c_2bin       = GP(44)* IkBbIKK	* NFkB ;
    flux_a_c_2ein       = GP(45)* IkBeIKK	* NFkB ;

    flux_d_c_2ain       = GP(46)* IkBaIKKNFkB ;
    flux_d_c_2bin       = GP(47)* IkBbIKKNFkB ;
    flux_d_c_2ein       = GP(48)* IkBeIKKNFkB ;

    flux_in_a           = GP(49)* IkBa ;
    flux_in_b           = GP(50)* IkBb ;
    flux_in_e           = GP(51)* IkBe ;
    
    flux_ex_a           = GP(52)* IkBan	;
    flux_ex_b           = GP(53)* IkBbn	;
    flux_ex_e           = GP(54)* IkBen	;
    
    flux_in_2an         = GP(55)* IkBaNFkB ;
    flux_in_2bn         = GP(56)* IkBbNFkB ;
    flux_in_2en         = GP(57)* IkBeNFkB ;  
    
    flux_ex_2an         = GP(58)* IkBaNFkBn	;
    flux_ex_2bn         = GP(59)* IkBbNFkBn	;
    flux_ex_2en         = GP(60)* IkBeNFkBn	;
    
    flux_in_n           = GP(61)* NFkB ;
    flux_ex_n           = GP(62)* NFkBn	;
    
    flux_pd_c_a         = GP(63)* IkBa ;
    flux_pd_c_b         = GP(64)* IkBb ;
    flux_pd_c_e         = GP(65)* IkBe ;
    
    flux_pd_n_a         = GP(66)* IkBan	;
    flux_pd_n_b         = GP(67)* IkBbn	;
    flux_pd_n_e         = GP(68)* IkBen	;    

    flux_pd_c_2an       = GP(69)* IkBaNFkB ;
    flux_pd_c_2bn       = GP(70)* IkBbNFkB ;
    flux_pd_c_2en       = GP(71)* IkBeNFkB ;
    
    flux_pd_n_2an       = GP(72)* IkBaNFkBn	;
    flux_pd_n_2bn       = GP(73)* IkBbNFkBn	;
    flux_pd_n_2en       = GP(74)* IkBeNFkBn	;
    
    flux_pd_c_2ai       = GP(75)* IkBaIKK  * ikk_multiplier;
    flux_pd_c_2bi       = GP(76)* IkBbIKK  * ikk_multiplier;
    flux_pd_c_2ei       = GP(77)* IkBeIKK  * ikk_multiplier;
    
    flux_pd_c_3ain      = GP(78)* IkBaIKKNFkB * ikk_multiplier;
    flux_pd_c_3bin      = GP(79)* IkBbIKKNFkB * ikk_multiplier;
    flux_pd_c_3ein      = GP(80)* IkBeIKKNFkB * ikk_multiplier;
    
    %new fluxes for -> p100t module
    %p100t synthesis
    flux_p100_synth       = + GP(82) + GP(81)* NFkBn ^ 3 ;
    
    
    %p100t -> p100
    flux_p100_trans       = GP(83) * p100t;
    
    %p100t->
    flux_p100t_deg = GP(87)*p100t;
    
    %p100->
    flux_p100_deg = GP(88)*p100;
    
    %p100 to p52
    if(p100==0||NIK==0)
       flux_p100_processing = 0;
    else
        flux_p100_processing  = (GP(84) * p100 * NIK)/(GP(85)+p100);
    end
    
    %p52 degradation
    flux_p52_deg          =(GP(86) * p52);

    %RelB p52 binding
    flux_RelBp52_binding        = (RelB * p52 * GP(94));
    
    %RelB p52 unbinding
    flux_RelBp52_unbinding      = (RelBp52 * GP(95));
    
    %RelBp52 degradation
    flux_RelBp52_deg            = (RelBp52 * GP(96));
    
    %RelBnp52n degradation
    flux_RelBnp52n_deg          = (RelBnp52n * GP(96));
    
    %RelBp52 import
    flux_RelBnp52n_import       = (RelBp52 * GP(97));
    
    %RelBnp52n export
    flux_RelBnp52n_export       = (RelBnp52n * GP(98));
    
    %RelBt expression rate
    flux_RelB_synth       = + GP(90) + GP(89)* NFkBn ^ 3 ;
    
    %RelBt->RelB
    flux_RelB_trans       = GP(91) * RelBt;
    
    %RelBt->
    flux_RelBt_deg = GP(92)*RelBt;
    
    %RelB->
    flux_RelB_deg = GP(93)*RelB;
    
    

    

    %%% *******************************
    %%% ***** TODO FINISH THIS!!!!*****
    %%% *******************************
    
%==========================================================
% Add/Subtract Fluxes to Concentrations

%%% constitutive txn    ==> IkB mRNA
       delta_IkBat      = delta_IkBat + flux_rsu_a;
       delta_IkBbt      = delta_IkBbt + flux_rsu_b;
       delta_IkBet      = delta_IkBet + flux_rsu_e;

%%% inducible txn       ==> IkB mRNA
       delta_IkBat      = delta_IkBat + flux_rsr_an;
       delta_IkBbt      = delta_IkBbt + flux_rsr_bn;
       delta_IkBet      = delta_IkBet + flux_rsr_en;

%%% mRNA Degradation    IkB mRNA ==> 
       delta_IkBat      = delta_IkBat - flux_rd_a;
       delta_IkBbt      = delta_IkBbt - flux_rd_b;
       delta_IkBet      = delta_IkBet - flux_rd_e;

%%% Translation         IkB mRNA ==> IkB mRNA + IkB     
       delta_IkBat      = delta_IkBat - flux_ps_c_a;
       delta_IkBat      = delta_IkBat + flux_ps_c_a;
       delta_IkBa       = delta_IkBa  + flux_ps_c_a;

       delta_IkBbt      = delta_IkBbt - flux_ps_c_b;
       delta_IkBbt      = delta_IkBbt + flux_ps_c_b;
       delta_IkBb       = delta_IkBb  + flux_ps_c_b;
       
       delta_IkBet      = delta_IkBet - flux_ps_c_e;       
       delta_IkBet      = delta_IkBet + flux_ps_c_e;
       delta_IkBe       = delta_IkBe  + flux_ps_c_e;

%%% IkB:NFkB Association  IkB+NFkB ==> IkBNFkB  , cyt 
       delta_IkBa       = delta_IkBa     - flux_a_c_an;
       delta_NFkB       = delta_NFkB     - flux_a_c_an;
       delta_IkBaNFkB   = delta_IkBaNFkB + flux_a_c_an;
       
       delta_IkBb       = delta_IkBb     - flux_a_c_bn;
       delta_NFkB       = delta_NFkB     - flux_a_c_bn;
       delta_IkBbNFkB   = delta_IkBbNFkB + flux_a_c_bn;
       
       delta_IkBe       = delta_IkBe     - flux_a_c_en;
       delta_NFkB       = delta_NFkB     - flux_a_c_en;
       delta_IkBeNFkB   = delta_IkBeNFkB + flux_a_c_en;

%%% IkB:NFkB Association  IkBn+NFkBn ==> IkBNFkBn , nuc
       delta_IkBan      = delta_IkBan     - flux_a_n_an;
       delta_NFkBn      = delta_NFkBn     - flux_a_n_an;
       delta_IkBaNFkBn  = delta_IkBaNFkBn + flux_a_n_an;
       
       delta_IkBbn      = delta_IkBbn     - flux_a_n_bn;
       delta_NFkBn      = delta_NFkBn     - flux_a_n_bn;
       delta_IkBbNFkBn  = delta_IkBbNFkBn + flux_a_n_bn;
       
       delta_IkBen      = delta_IkBen     - flux_a_n_en;
       delta_NFkBn      = delta_NFkBn     - flux_a_n_en;
       delta_IkBeNFkBn  = delta_IkBeNFkBn + flux_a_n_en;
       
%%% IkB:NFkB Dissociation  IkBNFkB ==> IkB+NFkB  , cyt 
       delta_IkBaNFkB   = delta_IkBaNFkB - flux_d_c_an;
       delta_IkBa       = delta_IkBa     + flux_d_c_an;
       delta_NFkB       = delta_NFkB     + flux_d_c_an;

       delta_IkBbNFkB   = delta_IkBbNFkB - flux_d_c_bn;
       delta_IkBb       = delta_IkBb     + flux_d_c_bn;
       delta_NFkB       = delta_NFkB     + flux_d_c_bn;

       delta_IkBeNFkB   = delta_IkBeNFkB - flux_d_c_en;
       delta_IkBe       = delta_IkBe     + flux_d_c_en;
       delta_NFkB       = delta_NFkB     + flux_d_c_en;

%%% IkB:NFkB Dissociation  IkBNFkBn ==> IkBn+NFkBn  , nuc
       delta_IkBaNFkBn  = delta_IkBaNFkBn - flux_d_n_an;
       delta_IkBan      = delta_IkBan     + flux_d_n_an;
       delta_NFkBn      = delta_NFkBn     + flux_d_n_an;
       
       delta_IkBbNFkBn  = delta_IkBbNFkBn - flux_d_n_bn;
       delta_IkBbn      = delta_IkBbn     + flux_d_n_bn;
       delta_NFkBn      = delta_NFkBn     + flux_d_n_bn;
       
       delta_IkBeNFkBn  = delta_IkBeNFkBn - flux_d_n_en;  
       delta_IkBen      = delta_IkBen     + flux_d_n_en;
       delta_NFkBn      = delta_NFkBn     + flux_d_n_en;

%%% IkB:IKK Association  IkB + IKK ==> IkBIKK , cyt
       delta_IkBa       = delta_IkBa    - flux_a_c_ai;
       delta_IKK        = delta_IKK     - flux_a_c_ai;
       delta_IkBaIKK    = delta_IkBaIKK + flux_a_c_ai;

       delta_IkBb       = delta_IkBb    - flux_a_c_bi;
       delta_IKK        = delta_IKK     - flux_a_c_bi;
       delta_IkBbIKK    = delta_IkBbIKK + flux_a_c_bi;
       
       delta_IkBe       = delta_IkBe    - flux_a_c_ei;
       delta_IKK        = delta_IKK     - flux_a_c_ei;
       delta_IkBeIKK    = delta_IkBeIKK + flux_a_c_ei;

%%% IkB:IKK Dissociation  IkBIKK ==> IkB + IKK , cyt
       delta_IkBaIKK    = delta_IkBaIKK - flux_d_c_ai;
       delta_IkBa       = delta_IkBa    + flux_d_c_ai;
       delta_IKK        = delta_IKK     + flux_d_c_ai;
       
       delta_IkBbIKK    = delta_IkBbIKK - flux_d_c_bi;
       delta_IkBb       = delta_IkBb    + flux_d_c_bi;
       delta_IKK        = delta_IKK     + flux_d_c_bi;

       delta_IkBeIKK    = delta_IkBeIKK - flux_d_c_ei;
       delta_IkBe       = delta_IkBe    + flux_d_c_ei;
       delta_IKK        = delta_IKK     + flux_d_c_ei;
       
%%% IkBNFkB:IKK Association  IkBNFkB+IKK==> IkBIKKNFkB
       delta_IkBaNFkB   = delta_IkBaNFkB    - flux_a_c_2ani;
       delta_IKK        = delta_IKK         - flux_a_c_2ani;
       delta_IkBaIKKNFkB= delta_IkBaIKKNFkB + flux_a_c_2ani;
       
       delta_IkBbNFkB   = delta_IkBbNFkB    - flux_a_c_2bni;
       delta_IKK        = delta_IKK         - flux_a_c_2bni;
       delta_IkBbIKKNFkB= delta_IkBbIKKNFkB + flux_a_c_2bni;
       
       delta_IkBeNFkB   = delta_IkBeNFkB    - flux_a_c_2eni;
       delta_IKK        = delta_IKK         - flux_a_c_2eni;
       delta_IkBeIKKNFkB= delta_IkBeIKKNFkB + flux_a_c_2eni;

%%% IkBNFkB:IKK Dissociation IkBIKKNFkB ==> IkBNFkB+IKK
       delta_IkBaIKKNFkB= delta_IkBaIKKNFkB - flux_d_c_2ani;
       delta_IkBaNFkB   = delta_IkBaNFkB    + flux_d_c_2ani;
       delta_IKK        = delta_IKK         + flux_d_c_2ani;
       
       delta_IkBbIKKNFkB= delta_IkBbIKKNFkB - flux_d_c_2bni;
       delta_IkBbNFkB   = delta_IkBbNFkB    + flux_d_c_2bni;
       delta_IKK        = delta_IKK         + flux_d_c_2bni;
       
       delta_IkBeIKKNFkB= delta_IkBeIKKNFkB - flux_d_c_2eni;
       delta_IkBeNFkB   = delta_IkBeNFkB    + flux_d_c_2eni;
       delta_IKK        = delta_IKK         + flux_d_c_2eni;     

%%% IkBIKK:NFkB Association  IkBIKK + NFkB ==> IkBIKKNFKB
       delta_IkBaIKK    = delta_IkBaIKK     - flux_a_c_2ain;
       delta_NFkB       = delta_NFkB        - flux_a_c_2ain;
       delta_IkBaIKKNFkB= delta_IkBaIKKNFkB + flux_a_c_2ain;
       
       delta_IkBbIKK    = delta_IkBbIKK     - flux_a_c_2bin;
       delta_NFkB       = delta_NFkB        - flux_a_c_2bin;
       delta_IkBbIKKNFkB= delta_IkBbIKKNFkB + flux_a_c_2bin;
       
       delta_IkBeIKK    = delta_IkBeIKK     - flux_a_c_2ein;
       delta_NFkB       = delta_NFkB        - flux_a_c_2ein;
       delta_IkBeIKKNFkB= delta_IkBeIKKNFkB + flux_a_c_2ein;

%%% IkBIKK:NFkB Dissociation IkBIKKNFKB ==> IkBIKK + NFkB 
       delta_IkBaIKKNFkB= delta_IkBaIKKNFkB - flux_d_c_2ain;
       delta_IkBaIKK    = delta_IkBaIKK     + flux_d_c_2ain;
       delta_NFkB       = delta_NFkB        + flux_d_c_2ain;
       
       delta_IkBbIKKNFkB= delta_IkBbIKKNFkB - flux_d_c_2bin;      
       delta_IkBbIKK    = delta_IkBbIKK     + flux_d_c_2bin;
       delta_NFkB       = delta_NFkB        + flux_d_c_2bin;
       
       delta_IkBeIKKNFkB= delta_IkBeIKKNFkB - flux_d_c_2ein;
       delta_IkBeIKK    = delta_IkBeIKK     + flux_d_c_2ein;
       delta_NFkB       = delta_NFkB        + flux_d_c_2ein;

%%% Free IkB Nuclear Import  IkB ==> IkBn
       delta_IkBa       = delta_IkBa  - flux_in_a;     
       delta_IkBan      = delta_IkBan + flux_in_a;
       
       delta_IkBb       = delta_IkBb  - flux_in_b;
       delta_IkBbn      = delta_IkBbn + flux_in_b;
       
       delta_IkBe       = delta_IkBe  - flux_in_e;
       delta_IkBen      = delta_IkBen + flux_in_e;
       
%%% Free IkB Nuclear Export  IkBn ==> IkB
       delta_IkBan      = delta_IkBan - flux_ex_a;
       delta_IkBa       = delta_IkBa  + flux_ex_a;
       
       delta_IkBbn      = delta_IkBbn - flux_ex_b;
       delta_IkBb       = delta_IkBb  + flux_ex_b;
       
       delta_IkBen      = delta_IkBen - flux_ex_e;
       delta_IkBe       = delta_IkBe  + flux_ex_e;
       
%%% Bound IkB Nuclear Import  IkBNFkB ==> IkBNFkBn
       delta_IkBaNFkB   = delta_IkBaNFkB  - flux_in_2an;
       delta_IkBaNFkBn  = delta_IkBaNFkBn + flux_in_2an;
       
       delta_IkBbNFkB   = delta_IkBbNFkB  - flux_in_2bn;
       delta_IkBbNFkBn  = delta_IkBbNFkBn + flux_in_2bn;
       
       delta_IkBeNFkB   = delta_IkBeNFkB  - flux_in_2en;
       delta_IkBeNFkBn  = delta_IkBeNFkBn + flux_in_2en;     
       
%%% Bound IkB Nuclear Export  IkBNFkBn ==> IkBNFkB
       delta_IkBaNFkBn  = delta_IkBaNFkBn - flux_ex_2an;
       delta_IkBaNFkB   = delta_IkBaNFkB  + flux_ex_2an;

       delta_IkBbNFkBn  = delta_IkBbNFkBn - flux_ex_2bn;
       delta_IkBbNFkB   = delta_IkBbNFkB  + flux_ex_2bn;
       
       delta_IkBeNFkBn  = delta_IkBeNFkBn - flux_ex_2en;
       delta_IkBeNFkB   = delta_IkBeNFkB  + flux_ex_2en;
       
%%% Free NFkB Shuttling  NFkB <==> NFkBn 
       delta_NFkB       = delta_NFkB  - flux_in_n;
       delta_NFkBn      = delta_NFkBn + flux_in_n;
      
       delta_NFkBn      = delta_NFkBn - flux_ex_n;  
       delta_NFkB       = delta_NFkB  + flux_ex_n;
       
%%% Free IkB Degradation  IkB ==>   , cyt       
       delta_IkBa       = delta_IkBa - flux_pd_c_a;
       delta_IkBb       = delta_IkBb - flux_pd_c_b;
       delta_IkBe       = delta_IkBe - flux_pd_c_e;         
       
%%% Free IkB Degradation  IkBn ==>   , nuc
       delta_IkBan      = delta_IkBan - flux_pd_n_a;
       delta_IkBbn      = delta_IkBbn - flux_pd_n_b;
       delta_IkBen      = delta_IkBen - flux_pd_n_e;

%%% Bound IkB Degradation  IkBNFkB ==> NFkB    , cyt
       delta_IkBaNFkB   = delta_IkBaNFkB - flux_pd_c_2an;
       delta_NFkB       = delta_NFkB     + flux_pd_c_2an;
       
       delta_IkBbNFkB   = delta_IkBbNFkB - flux_pd_c_2bn;
       delta_NFkB       = delta_NFkB     + flux_pd_c_2bn;
       
       delta_IkBeNFkB   = delta_IkBeNFkB - flux_pd_c_2en;
       delta_NFkB       = delta_NFkB     + flux_pd_c_2en;   
       
%%% Bound IkB Degradation  IkBNFkBn ==> NFkBn  , nuc
       delta_IkBaNFkBn  = delta_IkBaNFkBn - flux_pd_n_2an;
       delta_NFkBn      = delta_NFkBn     + flux_pd_n_2an;

       delta_IkBbNFkBn  = delta_IkBbNFkBn - flux_pd_n_2bn;
       delta_NFkBn      = delta_NFkBn     + flux_pd_n_2bn;       
       
       delta_IkBeNFkBn  = delta_IkBeNFkBn - flux_pd_n_2en;
       delta_NFkBn      = delta_NFkBn     + flux_pd_n_2en;
       
%%% IKK-mediated Free IkB Degradation  IkBIKK ==> IKK
       delta_IkBaIKK    = delta_IkBaIKK - flux_pd_c_2ai;
       delta_IKK        = delta_IKK     + flux_pd_c_2ai;       
 
       delta_IkBbIKK    = delta_IkBbIKK - flux_pd_c_2bi;
       delta_IKK        = delta_IKK     + flux_pd_c_2bi;

       delta_IkBeIKK    = delta_IkBeIKK - flux_pd_c_2ei;
       delta_IKK        = delta_IKK     + flux_pd_c_2ei;

%%% IKK-mediated Bound IkB Degradation IkBIKKNFKB ==> NFkB + IKK
       delta_IkBaIKKNFkB= delta_IkBaIKKNFkB - flux_pd_c_3ain;
       delta_IKK        = delta_IKK         + flux_pd_c_3ain;
       delta_NFkB       = delta_NFkB        + flux_pd_c_3ain;

       delta_IkBbIKKNFkB= delta_IkBbIKKNFkB - flux_pd_c_3bin;
       delta_IKK        = delta_IKK         + flux_pd_c_3bin;
       delta_NFkB       = delta_NFkB        + flux_pd_c_3bin;

       delta_IkBeIKKNFkB= delta_IkBeIKKNFkB - flux_pd_c_3ein;
       delta_IKK        = delta_IKK         + flux_pd_c_3ein;
       delta_NFkB       = delta_NFkB        + flux_pd_c_3ein;
       
%%% new Deltas for p100 p52 module
       
        delta_p100t=delta_p100t + flux_p100_synth - flux_p100t_deg - flux_p100t_deg;
        
        %delta p100
        delta_p100=delta_p100 + flux_p100_trans - flux_p100_processing - flux_p100_deg;

        %delta p52
        delta_p52 = delta_p52 + flux_p100_processing - flux_p52_deg;
        
        %delta RelB
        delta_RelB = delta_RelB + flux_RelB_trans - flux_RelB_deg - flux_RelBp52_binding + flux_RelBp52_unbinding;
        
        %delta RelBt
        delta_RelBt = delta_RelBt +flux_RelB_synth - flux_RelBt_deg;
        
        %delta RelBp52
        delta_RelBp52 = delta_RelBp52 + flux_RelBp52_binding - flux_RelBp52_unbinding - flux_RelBp52_deg - flux_RelBnp52n_import + flux_RelBnp52n_export;
        
        %delta RelBp52n
        delta_RelBnp52n = delta_RelBnp52n - flux_RelBnp52n_deg + flux_RelBnp52n_import - flux_RelBnp52n_export;
        
        %delta p52
        delta_p52 = delta_p52 - flux_RelBp52_binding + flux_RelBp52_unbinding;
        
        
        
%==========================================================
% Save Concentrations for the Next Time Step
     delta(1,1)      = delta_IkBa;
     delta(2,1)      = delta_IkBaIKK;
     delta(3,1)      = delta_IkBaIKKNFkB;
     delta(4,1)      = delta_IkBan;
     delta(5,1)      = delta_IkBaNFkB;
     delta(6,1)      = delta_IkBaNFkBn;
     delta(7,1)      = delta_IkBat;
     delta(8,1)      = delta_IkBb;
     delta(9,1)      = delta_IkBbIKK;
     delta(10,1)     = delta_IkBbIKKNFkB;
     delta(11,1)     = delta_IkBbn;
     delta(12,1)     = delta_IkBbNFkB;
     delta(13,1)     = delta_IkBbNFkBn;
     delta(14,1)     = delta_IkBbt;
     delta(15,1)     = delta_IkBe;
     delta(16,1)     = delta_IkBeIKK;
     delta(17,1)     = delta_IkBeIKKNFkB;
     delta(18,1)     = delta_IkBen;
     delta(19,1)     = delta_IkBeNFkB;
     delta(20,1)     = delta_IkBeNFkBn;
     delta(21,1)     = delta_IkBet;
     delta(22,1)     = delta_IKK;
     delta(23,1)     = delta_NFkB;
     delta(24,1)     = delta_NFkBn;
     delta(25,1)     = delta_p100t;
     delta(26,1)     = delta_p100;
     delta(27,1)     = delta_p52;
     delta(28,1)     = 0; %delta NIK
      delta(29,1)     = delta_RelB;
      delta(30,1)     = delta_RelBp52;
      delta(31,1)     = delta_RelBnp52n;
    delta(32,1)       = delta_RelBt;