% NF-kB Signaling Model version 2.1
%
% Werner, S.L., Barken, D., Hoffmann, A. 
%  Stimulus-specificity of gene expression programs determined 
%  by temporal control of IKK activity. 
% Science, 309, pp.1857-61. 2005.
%
% Updated by Mitchell, S. for non-canonical pathway activity
%
% Signaling Systems Laboratory (PI: Dr. Alexander Hoffmann) 
% Department of Chemistry and Biochemistry 
% University of California at San Diego
% 
% Questions should be addressed to
% Dr. Alexander Hoffmann    ahoffmann@ucsd.edu
%
%==========================================================
% Copyright 2007   The Regents of the University of California 
% All Rights Reserved 
%
% Permission to use, copy, modify and distribute any part of this
% NF-kB Signaling Model version 2.1 software for educational, research and
% non-profit purposes, without fee, and without a written agreement is
% hereby granted, provided that the above citation and copyright notice, this 
% paragraph and the following three paragraphs appear in all copies.
% 
% Those desiring to incorporate this NF-kB Signaling Model version 2.1
% software into commercial products or use for commercial purposes should
% contact the Technology Transfer & Intellectual Property Services,
% University of California, San Diego, 9500 Gilman Drive, Mail Code 0910,
% La Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345,
% E-MAIL:invent@ucsd.edu.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
% INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS NF-KB SIGNALING
% MODEL VERSION 2.1 SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
% ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% THE NF-KB SIGNALING MODEL VERSION 2.1 SOFTWARE PROVIDED HEREIN IS ON AN
% "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
% PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
% THE UNIVERSITY OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO
% WARRANTIES OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
% PARTICULAR PURPOSE, OR THAT THE USE OF THE NF-KB SIGNALING MODEL VERSION
% 2.1 SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.
%==========================================================

%==========================================================
% INSTRUCTIONS
%
% This model was built and tested using MatLab version 2006a.
%   Future versions of MatLab should run the model without modification.
%   Previous versions of MatLab have not been tested with this model.
%
% No MatLab toolboxes are required to run the model. 
%
% The model is contained in 3 MatLab files
%   nfkb_run_21pr.m
%   nfkb_sim_21pr.m
%   nfkb_ode_21pr.m
%
% To run the model: 
%    type 'nfkb_run_21pr' (without quotes) into the command line and hit enter
% 
% To display the results of the simulation, a plotting function is included in the sim file
%==========================================================


% RUN Function

function nfkb_run_21pr()

    % Global Variables
    global PLOT_FLAG START_TIME SIM_TIME;
            
        PLOT_FLAG       = 1;    % 1 = plot graphs, 0 = no graphs
        SIM_TIME        = 360;  % minutes in stimulation phase
        START_TIME      = -4000;% size of each equilbrium state iteration (don't change)

    % Generate IKK Input Curve 
    ikk_curves      = {'TNFp15' 'TNFc' 'LPSp45' 'basal_1'};
    ikk_curve_num   = 4;  % index of ikk curve to use (1 ... 4)
    
    % Set Simulation Variables - rate constants and initial concentrations
    params     = get_params();
    initvalues = get_init_concentrations();    
    NFkBMatrix = zeros(21,21);
    p52Matrix = zeros(21,21);
    RelBnp52nMatrix = zeros(21,21);
    RelBMatrix = zeros(21,21);
    IKKMatrix= zeros(21,21);
    NIKMatrix= zeros(21,21);
    
    IKKRange=[0:20];
    NIKRange=[0:5:100];
    for i=1:21
        for j=1:21
            %set NIK value to j
            initvalues(28)=NIKRange(j);

            fprintf 'NIK'
            initvalues(28)
            
            fprintf 'IKK:'
             %set IKK value to i
            generate_ikk(ikk_curves{ikk_curve_num},IKKRange(i));
            
            SIM_TIME        = 360;  % minutes in stimulation phase
            START_TIME      = -4000;% size of each equilbrium state iteration (don't 
            % Call NFkB_Sim File 
            steadyState=nfkb_sim_21pr(params, initvalues);
            
            fprintf('NIK:%d\n ',steadyState(28));
            fprintf('NFkB:%d\n',steadyState(24));
            fprintf('p52:%d\n',steadyState(27));
            fprintf('RelB:%d\n',steadyState(29));
            
            
            %NFkBSteadyState
            NFkBMatrix(i,j)=steadyState(24);
            %p52 steady state
            p52Matrix(i,j)=steadyState(27);
            %RelB p52 steady state
            RelBnp52nMatrix(i,j)=steadyState(31);
            %RelB steady state
            RelBMatrix(i,j)  = steadyState(29);
            %Input matrices for tracking
            IKKMatrix(i,j)=IKKRange(i);
            NIKMatrix(i,j)=NIKRange(j);
        end
    end
   csvwrite(strcat(num2str(IKKRange(i)),'xIKK',num2str(NIKRange(j)),'xNIK-','NFkBMatrix'),NFkBMatrix);
   csvwrite(strcat(num2str(IKKRange(i)),'xIKK',num2str(NIKRange(j)),'xNIK-','p52Matrix'),p52Matrix);
   csvwrite(strcat(num2str(IKKRange(i)),'xIKK',num2str(NIKRange(j)),'xNIK-','RelBnp52nMatrix'),RelBnp52nMatrix);
   csvwrite(strcat(num2str(IKKRange(i)),'xIKK',num2str(NIKRange(j)),'xNIK-','RelBMatrix'),RelBMatrix);  
   csvwrite(strcat(num2str(IKKRange(i)),'xIKK',num2str(NIKRange(j)),'xNIK-','IKKInput'),IKKMatrix);
   csvwrite(strcat(num2str(IKKRange(i)),'xIKK',num2str(NIKRange(j)),'xNIK-','NIKInput'),NIKMatrix);


end

%==========================================================
% Set the Reaction Parameters

function p = get_params()

p = zeros(98,1);

% Synthesis Reactions
%          ikba         ikbb        ikbe    
p(1:3)  = [1.848e-4;    4.272e-5;   3.048e-5;];% constitutive transcription
p(4:6)	= [7.92;        0.0;        0.8;    ]; % inducible transcription 
p(7:9)	= [0;           45;         45;     ]; % inducible txn delay
p(10:12)= [3.0;         3.0;        3.0;    ]; % Hill Coefficient 
p(13:15)= [.0168;       .0168;      .0168;  ]; % mRNA Degradation 
p(16:18)= [.2448;       .2448;      .2448;  ]; % translation rate

% Association/Dissociation Reactions
%          ikba         ikbb        ikbe   
p(19:21)= [30;          30;         30;     ]; % IkB:NFkB Association, cyt
p(22:24)= [30;          30;         30;     ]; % IkB:NFkB Association, nuc
p(25:27)= [6e-5;        6e-5;       6e-5;   ]; % IkB:NFkB Dissociation, cyt
p(28:30)= [6e-5;        6e-5;       6e-5;   ]; % IkB:NFkB Dissociation, nuc

p(31:33)= [1.35;        .36;        .54;    ]; % IkB:IKK Association, cyt
p(34:36)= [.075;        .105;       .105;   ]; % IkB:IKK Dissociation, cyt

p(37:39)= [11.1;        2.88;       4.2;    ]; % IkBNFkB:IKK Association, cyt
p(40:42)= [.075;        .105;       .105;   ]; % IkBNFkB:IKK Dissociation, cyt

p(43:45)= [30;          30;         30;     ]; % IkBIKK:NFkB Association, cyt
p(46:48)= [6e-5;        6e-5;       6e-5;   ]; % IkBIKK:NFkB Dissociation, cyt
 
% Import/Export Reactions
%          ikba         ikbb        ikbe    
p(49:51)= [.09;         .009;       .045;   ]; % Free IkB import
p(52:54)= [.012;        .012;       .012;   ]; % Free IkB export
p(55:57)= [.276;        .0276;      .138;   ]; % IkBNFkB import
p(58:60)= [.828;        .414;       .414;   ]; % IkBNFkB export
p(61)   = 5.4;                                 % Free NFkB import
p(62)   = 0.0048;                              % Free NFkB export

% Protein Degradation Reactions
%          ikba         ikbb        ikbe   
p(63:65)= [.12;         .18;        .18;    ]; % Free IkB Degradation, cyt
p(66:68)= [.12;         .18;        .18;    ]; % Free IkB Degradation, nuc

p(69:71)= [6e-5;        6e-5;       6e-5;   ]; % Bound IkB Degradation, cyt
p(72:74)= [6e-5;        6e-5;       6e-5;   ]; % Bound IkB Degradation, nuc

p(75:77)=[.0018;        .0006;      .0012;  ]; % IKK-mediated Free IkB Deg, cyt
p(78:80)=[.36;          .12;        .18;    ]; % IKK-mediated Bound IkB Deg, cyt
   

% TEST of free IkB deg by IKK being equal to bound IkB deg
p(75:77)=p(78:80);
p(31:33)=p(34:36);

%new parameters to correctly model the p100 module
%p100t inducible transcription rate
p(81)=1;
%p100t constituitive transcription rate
p(82)=0.0000008;

%p100 translation rate
p(83)=0.2;

%NIKKcat
p(84)=0.05;
%NIKKm
p(85)=10;

%p52 deg rate
p(86)=0.0228;

%p100t degradation rate
p(87)=0.00096;

%p100 degradation rate
p(88) = 0.0228;


%RelBt inducible transcription rate
p(89)=1;

%RelBt constituitive transcription rate
p(90)= 0.0000042;

%RelBt translation rate
p(91)=0.2;

%RelBt degradation rate
p(92)=0.00288;

%RelB Degradation Rate
p(93) = 0.0228;

%RelB p52 Binding Rate
p(94) = 0.00096;

%RelB p52 Unbinding Rate
p(95) = 0.0144;

%RelBp52 Degradation Rate
p(96) = 0.00024;

%RelBp52 Import
p(97) = 5.4;

%RelBp52 Export
p(98) = 0.0048;



end

%==========================================================
% Set the initial concentrations.
%    NFkB and IKK should always remain constant (0.1uM each)

function initvalues = get_init_concentrations()

 initvalues             = zeros(32,1);
 initvalues(1)          = 0.0;      % IkBa
 initvalues(2)          = 0.0;      % IkBaIKK
 initvalues(3)          = 0.0;      % IkBaIKKNFkB
 initvalues(4)          = 0.0;      % IkBan
 initvalues(5)          = 0.0;      % IkBaNFkB
 initvalues(6)          = 0.0;      % IkBaNFkBn
 initvalues(7)          = 0.0;      % IkBat
 initvalues(8)          = 0.0;      % IkBb
 initvalues(9)          = 0.0;      % IkBbIKK
 initvalues(10)         = 0.0;      % IkBbIKKNFkB
 initvalues(11)         = 0.0;      % IkBbn
 initvalues(12)         = 0.0;      % IkBbNFkB 
 initvalues(13)         = 0.0;      % IkBbNFkBn
 initvalues(14)         = 0.0;      % IkBbt
 initvalues(15)         = 0.0;      % IkBe
 initvalues(16)         = 0.0;      % IkBeIKK
 initvalues(17)         = 0.0;      % IkBeIKKNFkB
 initvalues(18)         = 0.0;      % IkBen
 initvalues(19)         = 0.0;      % IkBeNFkB
 initvalues(20)         = 0.0;      % IkBeNFkBn
 initvalues(21)         = 0.0;      % IkBet
 initvalues(22)         = 0.1;      % IKK
 initvalues(23)         = 0.0;      % NFkB
 initvalues(24)         = 0.1;      % NFkBn
 initvalues(25)         = 0.0;      % p100t
 initvalues(26)         = 0.0;      % p100
 initvalues(27)         = 0.0;      % p52
 initvalues(28)         = 0.0;      % NIK
 initvalues(29)         = 0.0;      % RelB
 initvalues(30)         = 0.0;      % RelBp52
 initvalues(31)         = 0.0;      % RelBnp52n
 initvalues(32)         = 0.0;      % RelBt
 
 
end

% Generate an IKK input curve using lists of fraction active and timepoints
%   This function utilizes the MatLab Curve Fitting Toolbox
%   If you do not have this toolbox, you can modify this to use interp1()

function generate_ikk(stimulus,level)

    global CURRENT_IKK;
    FT = fittype('pchipinterp'); % Fit-Type Library, set in Run File
    
    if (strcmp(stimulus,'TNFp15'))
        values = [1 60 100 65 50 36 21 16 10 1 1 1] / 100;
        time = [0 5 10 15 20 25 30 45 59 60 120 360];
    elseif (strcmp(stimulus,'TNFc'))
        values = [1 60 100 65 50 36 30 30 30] /100; 
        time = [0 5 10 15 20 25 30 60 360];
    elseif (strcmp(stimulus,'LPSp45')) 
        values = [1 3 8 24 25 15 8 5] / 100;
        time = [0 15 30 45 60 120 240 360];
    elseif (strcmp(stimulus,'basal_1'))
        values = [level level] /100;
        time = [0 360];
    else
        error('nfkb_run_21pr:generate_ikk() Incorrect IKK input name');
    end
         disp(['IKK ' stimulus]);
         time = time(:);
         values = values(:);
         ok_ = ~(isnan(time) | isnan(values));
         %CURRENT_IKK = interp1(time, values, 0:time(end),'pchip',values(end));
         CURRENT_IKK = fit(time(ok_),values(ok_),FT);
         CURRENT_IKK(0)
end